import json
import time
import sys
import os

from main.libs.CFunc import CFunc
from main.jsonrpc.CJsonRpc import CJsonRpc
from datetime import datetime

class CKodi(object):
	
	def __init__(self):
		self.iPlayerId = 1
		self.iZwischenprogrammPlaylistId = 1
		self.sJsonRpcVersion = '2.0'
		self.objJsonRpc = CJsonRpc()
		self.bIsZwischenprogrammRunnung = False
		self.iZwischenprogrammItem = -1
		self.dZwischenprogrammProzent = -1
		self.bIsProgrammRunnung = False
	
	def SetConfig(self, objCConfig):
		self.config = objCConfig
		self.objJsonRpc.sUrl = self.config.GetValue('KODI', 'adresse')
		self.objJsonRpc.objHeader = self.CreateHeader()
		
	def SaveZwischenprogrammPosition(self):
		jsonProperties = self.PlayerGetProperties()
		JsonRpc = CJsonRpc()
		if(JsonRpc.IsResponseValid(jsonProperties)):
			arProperties = JsonRpc.GetResultObject(jsonProperties)
			self.iZwischenprogrammItem = arProperties['position']
			self.dZwischenprogrammProzent = arProperties['percentage']
			
	def CreateHeader(self):
		sUser = self.config.GetValue('KODI', 'user')
		sPasswort = self.config.GetValue('KODI', 'passwort')
		sBase64Enc = CFunc.CreateHttpBasicAuth(sUser, sPasswort)
		return { 'Content-Type': 'application/json', 'Authorization': sBase64Enc }
		
	def PlaylistGetPlaylists(self):
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Playlist.GetPlaylists", "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def Ping(self):
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "JSONRPC.Ping", "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayerGetProperties(self):
		objParameter = {"playerid": self.iPlayerId, "properties": ["percentage", "playlistid", "position","repeat","time","totaltime", "currentvideostream"] }
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.GetProperties", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
		
	def PlaylistGetItems(self):
		objParameter = {"playlistid": self.iZwischenprogrammPlaylistId }
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Playlist.GetItems", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayerGetActivePlayers(self):
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.GetActivePlayers", "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayerOpen(self, objParameter):
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.Open", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayElement(self, arPlaylistElement):
		objPlaylistElement = arPlaylistElement['ELEMENT']
		if(objPlaylistElement.sTyp == "PLAY.STREAM"):
			objParameter = {"item": { "file": objPlaylistElement.sUrl} }
			self.PlayerOpen(objParameter)
	
	def PlayerSeek(self, objParameter):
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.Seek", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayerStop(self, objParameter):
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.Stop", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def IsKodiAktiv(self):
		bIsAktiv = False
		bCheckFertig = False
		iMaxVersuche = 100
		iVersuch =0
		iTimeout = 2
		# {"id":1,"jsonrpc":"2.0","result":"pong"}'
		while(bCheckFertig == False):
			jsonResponse = self.Ping()
			arResult = json.loads(jsonResponse.decode("utf-8"))
			if('result' in arResult and arResult["result"] == 'pong'):
				bCheckFertig = True
				bIsAktiv = True
			else:
				time.sleep(iTimeout)
				
			iVersuch+=1
			if(iVersuch >= iMaxVersuche):
				bCheckFertig = True
				
		return bIsAktiv
	
	def IsPlayerAktiv(self):
		bIsAktiv = False
		jsonResponse = self.PlayerGetActivePlayers()
		arResponse = json.loads(jsonResponse.decode('utf-8'))
		arResult = arResponse["result"];
		
		if(isinstance(arResult, list) and len(arResult) > 0):
			if('playerid' in arResult[0] and arResult[0]["playerid"] > 0):
				bIsAktiv = True
		
		return bIsAktiv
		
	def FilesGetDirectory(self, objPlaylistElement):
		objParameter = { "directory": objPlaylistElement.sPfad,  "media": "video", "sort": {"order": "descending", "method": "file"} }
		
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Files.GetDirectory", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlaylistAdd(self, objFile):
		arFile = { "file": objFile["file"] }
		objParameter = { "item": arFile,  "playlistid": self.iZwischenprogrammPlaylistId }
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Playlist.Add", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayerSetVideoStream(self, iNumber):
		objParameter = { "stream": iNumber,  "playerid": self.iPlayerId }
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.SetVideoStream", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def PlayerOpenPlaylist(self, iPlaylistId):
		objParameter = {"item": { "playlistid": iPlaylistId, "movieid":0}, "options":{"repeat":"all"} }
		objData = { "jsonrpc": self.sJsonRpcVersion, "method": "Player.Open", "params": objParameter, "id": self.iPlayerId }
		self.objJsonRpc.sData = json.dumps(objData).encode('utf-8')
		return self.objJsonRpc.SendAndReceive()
	
	def StartZwischenprogramm(self, iPlaylistId = None):
		
		iListId = self.iZwischenprogrammPlaylistId;
		if(iPlaylistId):
			iListId = iPlaylistId;
		objParameter = {"item": { "playlistid": self.iZwischenprogrammPlaylistId }, "options":{"repeat":"all"} }
		self.PlayerOpen(objParameter)
		self.bIsZwischenprogrammRunnung = True
	
	def LoadZwischenprogramm(self, arPlaylist):
		
		for arSingleElement in arPlaylist:
			
			objPlaylistElement = arSingleElement['ELEMENT']
			
			if(objPlaylistElement.sTyp == 'PLAY.FOLDER'):
				objResponse = self.FilesGetDirectory(objPlaylistElement)
				
				if(self.objJsonRpc.IsResponseValid(objResponse)):
					objResult = self.objJsonRpc.GetResultObject(objResponse)
					for objFile in objResult['files']:
						self.PlaylistAdd(objFile)
					
	def GetProgramm4Now(self, arPlaylist):
		sFormat = '%H:%M:%S'
		objPlaylistElement = None
		for arSingleElement in arPlaylist:
			dtZEITVON = datetime.strptime(arSingleElement["ZEITVON"], sFormat).time()
			dtZEITBIS = datetime.strptime(arSingleElement["ZEITBIS"], sFormat).time()
			dtJetzt = datetime.now().time()
			if(dtJetzt >= dtZEITVON and dtJetzt <=dtZEITBIS):
				objPlaylistElement = arSingleElement
				
		return objPlaylistElement
